;;; bookmark-conf.el --- functions for bookmarking
;;;
;;; commentary:
;;; Emacs' built-in bookmark facility allows for 1 single
;;; bookmark file by default.
;;; I modify it to open different files.
;;;
;;; bmark-add    - save current buffer to a bookmark file
;;; bmark-open   - go to a specific bookmark file
;;; bmark-dir    - directory to keep bookmark files
;;; bmark-file   - concat dir to filename based on assoc
;;; bmark-list   - chars and labels for bookmark files
;;; list-options - list all options in an alist
;;; list-keys    - just list keys in an alist
;;;
;;; My keybinds:
;;; - C-x b      - bookmark list
;;; - C-x B      - bookmark add
;;;
;;; code:

(require 'bookmark)

;; the default directory for your bookmark files.
(defvar bmark-dir "~/.config/emacs/bookm")

;; alist of characters pointing to filenames, relative to 'bmark-dir'.
(defvar bmark-list  '(("o" . "online")
		      ("l" . "local")
		      ("r" . "random")
		      ("e" . "emacs")
		      ("c" . "computers")
		      ("b" . "bookmarks")))

(defun list-keys (lst)
  "List `car x` for all X in LST."
  (mapcar #'car lst))

(defun char-prompt ()
  "Prompt the user for a single char, list acceptable chars."
  (char-to-string (read-char (format "Key %s: " (list-keys bmark-list)))))

(defun bmark-file (key)
  "Filename KEY in `bmark-dir`."
  (concat bmark-dir "/" (cdr (assoc key bmark-list))))

(defun bmark-save ()
  "Save the current active bookmark file."
  (interactive)
  (bookmark-save nil bookmark-default-file 0))

(defun bmark-quit ()
  "Save and close the bookmark window."
  (interactive)
  (bmark-save)
  (quit-window))

(defun bmark-init (file)
  "Set the active bookmark file to FILE and load it (overwriting the original)."
  (interactive)
  (setq bookmark-default-file file)
  (if (file-exists-p file)
      (bookmark-load file t t t)))

;; TODO: More elegant popup (like org-capture)
(defun bmark-add (key)
  "Save the current page as a bookmark in (assoc KEY `bmark-list`)."
  (interactive (list (char-prompt)))
  (let ((file (bmark-file key)))
    (bmark-init file) ;; alist '() -> load -> bookmark-file
    (bookmark-set)
    (bmark-save)))

(defun bmark-open (key)
  "Open a bookmark file based on char input KEY."
  (interactive (list (char-prompt)))
  (let ((file (bmark-file key)))
    (bmark-init file)
    (bookmark-bmenu-list)
    (switch-to-buffer "*Bookmark List*")))

(use-package bookmark
  :bind (:map bookmark-bmenu-mode-map
	      ("q" . bmark-quit)
	      ("s" . bmark-save)))

(provide 'bookmark-conf)

;;; bookmark-conf.el ends here
