;;; ibuffer-conf --- ibuffer configurations
;;;
;;; commentary:
;;; I use filter groups like:
;;; -[Net]   -- EWW, Elpher, IRC
;;; -[Dired] -- Directory listings
;;; -[Stars] -- Like *Scratch* and *Message*
;;;
;;; TODO:
;;; - Hide filter groups except when in a certain set.
;;; - Make adding to the "home" set easier.
;;; - There are many flymake warnings but ehhhhhhhhhh.
;;;
;;; code:

;; Groups to hide entirely.
(defvar hidden-groups '("Hidden"))

;; Main package declaration
(use-package ibuffer
  :bind (("C-c i" . ibuffer)
	 :map ibuffer-mode-map
	 ("M-<RET>" . ibuffer-toggle-all)
	 ("#"       . ibuffer-switch-format)
	 ("h"       . ibuffer-toggle-current))
  :config

(defun ibuffer-toggle-current ()
  "Collapse or expand the current filter group.
If point is on a buffer name, expand the group it is in."
  (interactive)
  (unless (char-equal (char-after) ?\[)
    (ibuffer-backward-filter-group))
  (ibuffer-toggle-filter-group))
  
  (defun ibuffer-toggle-all ()
    "Collapse / expand ALL filters in ibuffer."
    (interactive)
    (if ibuffer-hidden-filter-groups
	(setq ibuffer-hidden-filter-groups '())
      (setq ibuffer-hidden-filter-groups
	    (mapcar #'car (ibuffer-current-filter-groups-with-position))))
    (ibuffer-update nil t))

  ;; Two formats (toggle with #)
  ;; one for wider displays, the other for split windows.
  (setq ibuffer-formats
	'((mark modified read-only locked " "
		(name 18 18 :left :elide) " "
		(size 9 -1 :right) " "
		(mode 16 16 :left :elide) " "
		(filename-and-process 70 70 :right :elide))

	  (mark modified " "
		(name 16 -1)
		" " mode)))

  (setq ibuffer-expert t)


;; (defvar lol '("Org" (filename ".el")))
(defun ibuffer-add (form)
 (push form (cdar ibuffer-saved-filter-groups)))
       
  (setq ibuffer-saved-filter-groups
	'(("home"
	   ("Emacs" (and (filename         . "config/emacs*")
			 (visiting-file)))
	   ("Prog"  (and (filename         . "/lin/prog*")
			 (visiting-file)))
	   ("Org"   (or  (file-extension    . "org")
			 (derived-mode      . org-mode)
			 (derived-mode      . org-agenda-mode)))
	   ("Mail"  (or  (derived-mode      . rmail-mode)
			 (derived-mode      . message-mode)))
	   ("Gnus"  (or  (derived-mode      . gnus-mode)
			 (saved             . "gnus")))
	   ("Net"   (or  (mode              . eww-mode)
			 (derived-mode      . rcirc-mode)
			 (mode              . elpher-mode)))
	   ("Music"      (name              . "*MPC"))
	   ("Dired" (or  (derived-mode      . dired-mode)
			 (derived-mode      . image-mode)))
	   ("Proc"       (process))
	   ("Stars"      (name              . "[*]")))))
  ;; starred-name does not catch all starred buffers, uh ok

  :hook (ibuffer-mode . (lambda ()
	  (ibuffer-auto-mode 1)
	  (ibuffer-switch-to-saved-filter-groups "home")
	  (setq ibuffer-never-show-predicates '("*async-native" "*Completion"))
	  (setq ibuffer-hidden-filter-groups '("Stars"))
	  (add-to-list 'ibuffer-saved-filters
		       '("modified" (and (modified)
					 (not  (starred-name))))))))

;; additional ibuffer functionality
(use-package ibuffer-ext
  :custom
  (ibuffer-show-empty-filter-groups nil)
  (ibuffer-old-time 12))

(defun hide-this-buffer ()
"Hide the currently selected buffer."
  (interactive)
  (add-to-list 'ibuffer-never-show-predicates
	       (buffer-name (ibuffer-current-buffer)))
  (ibuffer-update nil))

(keymap-global-set "C-c h" 'hide-this-buffer)

(defun car-to-last (lst)
"Move the first element of LST to the last position."
  (append (cdr lst) (list (car lst))))

(defun omit-hidden-groups (lst)
  "Remove all items in `hidden-groups` from LST."
  (dolist (i hidden-groups)
    (setq lst (assoc-delete-all i lst)))
  lst)

;; Apply the function ibuffer's list generator
(advice-add 'ibuffer-generate-filter-groups
	    :filter-return #'omit-hidden-groups)

;; Make 'Default' the topmost.
(advice-add 'ibuffer-generate-filter-groups
	    :filter-return #'car-to-last)

(provide 'ibuffer-conf)

;;; ibuffer-conf.el ends here.
